#!/usr/bin/python
# Geek Credit pocket script v0.019
# (C) 2004 by Alexander Komarov < toor aT izardsnest.org >
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License, or 
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#    Intel hereby disclaims all copyright interest in the program
#    'Geek Credit' written by Alexander Komarov.
#    F. Thomas Dunlap, Jr.
#    Vice President, General Counsel and Secretary
#    INTEL CORPORATION
#    19 Mar 04

import getopt, sys, string
import GeekCredit

def usage () :
    # Example :
    # GkPocket.py --GetCredits=2,John_Smith,Alexander_Komarov_[Izard]_<izard@izardsnest.org> -agc.cfg
    # GkPocket.py -acredits.txt
    print "Usage: GcPocket.exe with one of commands:"
    print "-i or --GetIssuers to get the list of the GCs issuers we can pay"
    print "-s or --GetStats gets the list of the GCs issuers and number of "
    print "Geek Credits we have"
    # -fgk_bobby.cfg -ppassphrase -c3,catty_<catty@wonderland.ru>,alice_<alice@wonderland.ru>,bobby_<bobby@wonderland.ru>
    print "-c... or --GetCredits=number,seller,trusted_issuers to prepare"
    print "a number of Geek Credits to pay to seller issued by specified"
    print "users. The credits prepared are printed to stdout. Don't miss them,"
    print "Or you will have to move them from archive back to the pocket"
    print "All spaces should be replaced by underscores"
    # -fgk_catty.cfg -ato_catty.txt
    print "-afilename or --AcceptCredits=filename"
    print "-glist or --GetGood=list to get the issuers we accept from the list"
    print "option: -ffilename or --ConfigFile=filename to set the config file"
    
def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], "sig:c:a:f:", \
                    ["GetStats", "GetIssuers", "GetGood=", "GetCredits=", \
                     "AcceptCredits=", "ConfigFile="])
    except getopt.GetoptError:
        # print help information and exit:
        usage()
        sys.exit(2)
    if args or len(opts) == 0 :
        usage()
        sys.exit(2)
    
    for o, a in opts:
        if o in ("-f", "--ConfigFile"):
            filename = a
            break
        else :
            filename = 'gc.cfg'

# Create GeekCreditPocket object, read data from file
    try :
        pocket = GeekCredit.GeekCreditPocket(filename)
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        sys.exit(2)
    except Exception, inst:
        print "Unexpected error:" + str(inst.args)
        sys.exit(2)

    for o, a in opts:
        if o in ("-s", "--GetStats"):
            issuers = pocket.getCreditsStats()
            for i in issuers :
                print i
            sys.exit(0)
        if o in ("-i", "--GetIssuers"):
            issuers = pocket.getIssuers()
            for i in issuers :
                print i
            sys.exit(0)
        if o in ("-g", "--GetGood"):
            # All " " are replaced to "_" in input arguments,
            # replace them back
            a = string.replace(a, '_', ' ')
            issuers = string.split(a, ',')
            good = pocket.getGoodIssuers(issuers)
            for i in good :
                print i
            sys.exit(0)
        if o in ("-c", "--GetCredits"):
            # All " " are replaced to "_" in input arguments,
            # replace them back
            a = string.replace(a, '_', ' ')
            data = string.split(a, ',')

            # Number of Geek Credits to find
            number = int(data[0])
            if number < 1 :
                print 'Wrong number: ' + str(number)
                sys.exit(2)
            # User who will get those Geek Credits
            seller = data[1]
            # The list of the issuers this user trusts
            issuers = data[2:]
            myCredits, result = pocket.signCredits(seller, issuers, number)
            if len(myCredits) == 0 :
                print 'Failed to get credits' + result
                sys.exit(2)
            for credit in myCredits :
                print credit.getText()
            pocket.savePocket()
            sys.exit(0)
        if o in ("-a", "--AcceptCredits"):
            try :
                f = file(a)
                myCredits = f.read()
                f.close()
            except IOError, (errno, strerror):
                print "Can't read credits. I/O error(%s): %s" % (errno, strerror)
                sys.exit(2)
            except :
                print "Unexpected error:", sys.exc_info()[0]
                sys.exit(2)
            if not len(myCredits) :
                print "Can't read credits."
                sys.exit(2)
            creditsAccepted, orig, result = pocket.acceptCredits(myCredits)
            pocket.savePocket()
            if creditsAccepted > 0 :
                print 'Accepted ' + str(creditsAccepted) + ' from ' + str(orig)
            if len(result) > 0 :
                print result
            sys.exit(0)
    usage()
    sys.exit(2)

if __name__ == "__main__":
    main()

