#!/usr/bin/python
# Geek Credit pocket script v0.018
# (C) 2004 by Alexander Komarov < toor aT izardsnest.org >
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License, or 
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#    Intel hereby disclaims all copyright interest in the program
#    'Geek Credit' written by Alexander Komarov.
#    F. Thomas Dunlap, Jr.
#    Vice President, General Counsel and Secretary
#    INTEL CORPORATION
#    19 Mar 04

import sys
from Tkinter import *
import tkFileDialog, tkSimpleDialog, tkMessageBox, ScrolledText
import GeekCredit

cfgname = 'gc.cfg'
# Create GeekCreditPocket object, read data from file
try :
    pocket = GeekCredit.GeekCreditPocket(cfgname)
except IOError, (errno, strerror):
    tkMessageBox.showwarning("Unexpected error:",
                                    "I/O error(%s): %s" % (errno, strerror))
    sys.exit(2)
except Exception, inst:
    tkMessageBox.showwarning("Unexpected error:",
                                    str(inst.args))
    sys.exit(2)

class GetDialog(tkSimpleDialog.Dialog) :
    def body(self, master):
        Label(master, text="Number of Credits to generate:").grid(row=0)
        Label(master, text="Seller PGP ID:").grid(row=1)
        Label(master, text="Your PGP key passphrase:").grid(row=2)
        Label(master, text="Issuers list:").grid(row=3)
        self.e1 = Entry(master)
        self.e2 = Entry(master)
        self.e3 = Entry(master)

        self.e4 = Listbox(master, selectmode = MULTIPLE)
        issuers = pocket.getIssuers()
        for i in issuers :
            self.e4.insert(END, i)

        self.e1.grid(row=0, column=1)
        self.e2.grid(row=1, column=1)
        self.e3.grid(row=2, column=1)
        self.e4.grid(row=3, column=1)
        self.result = 0, "", "", []
        return self.e1 # initial focus
    def validate (self) :
        try :
            number = int(self.e1.get())
            second = self.e2.get()
            passphrase = self.e3.get()
            third = self.e4.curselection()
            if len(third) < 1 :
                raise
            res = []
            for i in third :
                res.append(self.e4.get(int(i)))
            self.result = number, second, passphrase, res
            return 1
        except :
            tkMessageBox.showwarning("Unexpected error:",
                                    'Wrong input')
            return 0
            
    def apply(self):
        pass

class ShowDialog(tkSimpleDialog.Dialog) :
    def __init__(self, parent, title, credits) :
        self.credits = credits
        tkSimpleDialog.Dialog.__init__(self, parent, title)

    def body(self, master):
        Label(master, text="Please copy & paste the credits below so \
you can send them to seller").grid(row=0)
        self.e1 = ScrolledText.ScrolledText(master)

        for i in self.credits :
            self.e1.insert(END, i)
        self.e1.grid(row=1)

        return self.e1 # initial focus
    def apply(self):
        pass

class Application(Frame):
    def getCredits(self):
        dlg = GetDialog(self)
        number, seller, passphrase, issuers = dlg.result
        if not number and not seller and not issuers :
            return
        if number < 1 :
            tkMessageBox.showwarning("Unexpected error:",
                                    'Wrong number: ' + str(number))
            return
        myCredits, result = pocket.signCredits(seller, issuers, number, passphrase)
        if len(myCredits) == 0 :
            tkMessageBox.showwarning("Unexpected error:",
                                    result)
            return
        pocket.savePocket()
        self.updateText()
        GlobalCredits=[]
        for credit in myCredits :
            GlobalCredits.append(credit.getText())
        ShowDialog(self, "Credits", GlobalCredits)
        self.GlobalCredits=[]

    def accCredits(self):
        filename = tkFileDialog.askopenfilename()
        if filename == "" :
            return
        try :
            f = file(filename)
            myCredits = f.read()
            f.close()
        except IOError, (errno, strerror):
            tkMessageBox.showwarning("Unexpected error:",
                                    "Can't read credits. I/O error(%s): %s" \
                                    % (errno, strerror))
            return
        except :
            tkMessageBox.showwarning("Unexpected error:",
                                    sys.exc_info()[0])
            return
        if not len(myCredits) :
            tkMessageBox.showwarning("Can't read credits.",
                                    sys.exc_info()[0])
            return
        creditsAccepted, orig, result = pocket.acceptCredits(myCredits)
        pocket.savePocket()
        if len(result) > 0 :
            tkMessageBox.showwarning("Warning:", result)
        self.updateText()

    def updateText(self) :
        stats = pocket.getCreditsStats()
        all = 0
        for i, j in stats :
            all = all + j
        self.credits.set(pocket.name + " | " + str(all))

    def createWidgets(self):
        stats = pocket.getCreditsStats()
        all = 0
        for i, j in stats :
            all = all + j
        self.credits = StringVar()
        self.txt = Label(self, textvariable=self.credits)
        self.txt.pack(fill=X, expand=1)
        self.credits.set(pocket.name + " | " + str(all))
        self.txt.pack({"side": "bottom"})
        self.txt2 = Label(self, text="User name:         | Geek Credits: ")
        self.txt2.pack(fill=X, expand=1)
        self.txt2.pack({"side": "bottom"})

        self.get = Button(self)
        self.get["text"] = "Get"
        self.get["fg"]      = "blue"
        self.get["command"] = self.getCredits
        self.get.pack({"side": "left"})

        self.acc = Button(self)
        self.acc["text"] = "Accept"
        self.acc["fg"]      = "blue"
        self.acc["command"] = self.accCredits
        self.acc.pack({"side": "left"})

        self.QUIT = Button(self)
        self.QUIT["text"]    = "QUIT"
        self.QUIT["fg"]      = "red"
        self.QUIT["command"] = self.quit
        self.QUIT.pack({"side": "left"})

    def __init__(self, master=None):
        Frame.__init__(self, master)
        self.pack()
        self.createWidgets()
 
if __name__ == "__main__":
    app = Application()
    app.master.title("Geek Credit pocket")
    app.master.minsize(240, 75)
    app.master.maxsize(280, 85)
    app.mainloop()

