# Geek Credit pocket reference code v0.019
# (C) 2004 by Alexander Komarov < toor aT izardsnest.org >
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
#    Intel hereby disclaims all copyright interest in the program
#    'Geek Credit' written by Alexander Komarov.
#    F. Thomas Dunlap, Jr.
#    Vice President, General Counsel and Secretary
#    INTEL CORPORATION
#    19 Mar 04

import sys, os, string, random, re
import sha, time
import ConfigParser
import tempfile

__author__  = 'Alexander Komarov'
__version__ = '0.019'

class GeekCredit :
    """
        The instance of this class is a Geek Credit
    """
    def __init__ (self, text, uid = 0, com = -1) :
        """
            Issuing a new Geek Credit if text == ""
            or creating from stream otherways
        """
        if text == "" :
            if com < 0 or uid == 0 :
                raise
            self.text = 'Geek Credit 0.3 ' + str(uid) + ' ' + str(com) + '\n'
            return

        # Or we are creating from stream
        strings = string.split(text, '\n')
        if not re.search('Geek Credit 0.3 \d{9} \d+', strings[0]) or \
           string.find(text, '-----BEGIN PGP SIGNATURE-----') == -1 :
            raise Exception('Bad Credit')
        # normalize the text, remove trailing '\n's
        for i in range (len(strings) - 2, 0, -1) :
            if strings[i] == "" :
                strings.pop(i)
            else :
                break
        self.text = string.join(strings, '\n')

    def signTransaction (self, pocket, seller, passPhrase) :
        """
            Adds payer and seller PGP IDs, date and
            signs resulting Geek Credit
        """

        # Add payer string
        self.text = self.text + pocket.name + '\n'
        # Add seller string and current time
        self.text = self.text + seller + '\n' + time.asctime() + '\n'
        # Add signature
        signature = gpgMakeSignature(sha.new(self.text).digest(), \
                                        pocket.name, passPhrase)
        if signature == "" :
            # Failed signing this Geek Credit
            raise
        self.text = self.text + signature + '\n'

    def getIssuer (self) :
        return string.split(self.text, '\n')[1]

    def getId (self) :
        return string.split(string.split(self.text, '\n')[0])[3]

    def getCommunity (self) :
        return string.split(string.split(self.text, '\n')[0])[4]

    def getLastName (self) :
        strings = string.split(self.text, '\n')
        for i in range(len(strings) - 1, 0, -1) :
            if strings[i] == "-----BEGIN PGP SIGNATURE-----" :
                return strings[i - 2]
                break
        print 'Did not find last name'
        return ""

    def findDifference (self, credit) :
        """
            Assuming that current one of this 2 Geek Credit is a counterfeit
            return the string with investigation result.
        """
        strings = string.split(self.text, '\n')
        strings2 = string.split(credit.text, '\n')
        k = 3
        first = True
        for i in range(4, len(strings) - 1) :
            if strings[i] == "-----BEGIN PGP SIGNATURE-----" :
                while k < len(strings2) - 1 :
                    k = k + 1
                    if strings2[k] == "-----BEGIN PGP SIGNATURE-----" :
                        # The if strings[i - 3] != strings2[k - 3] :
                        # case will be checked in the verifySignatures,
                        # as this would mean a stolen Geek Credit, not
                        # counterfeit
                        if strings[i - 2] != strings2[k - 2] :
                            if first :
                                # This is myself!!!
                                return "I am the attacker"
                            return strings[i - 3] + " is the attacker"
                        else :
                            lastName = strings[i - 3]
                        first = False
                        break
        return "No difference found, you are trying to accept the same \
                Geek Credit received from " + lastName

    def verifySignatures (self, keyserver) :
        """
            For each signature in this Geek Credit,
            veryfy the signature with GnuPG
            Last signature and issuer signature must be valid
            others should be wether valid or not checkable
            because we don't have a key (policy #6)
        """
        theRest = self.text
        lastSignature = True
        while True :
            result = ""
            strings = string.split(theRest, '-----BEGIN PGP SIGNATURE-----')
            if len(strings) < 2 :
                return "Bad Geek Credit"
            theRest = string.join(strings[:len(strings) - 1], \
                                    '-----BEGIN PGP SIGNATURE-----')
            temp = string.split(theRest, '\n')
            # Check if this Geek Credit was stolen
            if not lastSignature :
                if temp[len(temp) - 3] != name :
                    return "Geek Credit was stolen by " + name
            name = temp[len(temp) - 4]
            # Check if we have the public key for this person
            if gpgHasKey(name, keyserver) :
                if not gpgVerifySignature(theRest, strings[len(strings) - 1], \
                                            name) :
                    return "Signature of " + name + \
                                    " failed verification"
            else :
                # If there is no public key available and this is the last
                # transaction
                # Lets fail it, we can't veryfy the Geek Credit
                if lastSignature :
                    return "Failed verifying, no such key: " + name
                result = "Failed verifying, no such key: " + name
            trunc = string.rfind(theRest, '-----END PGP SIGNATURE-----')
            if -1 == trunc :
                # Exit the loop, this was the last signature
                break
            # Remove the last transaction data, it was verified successfully
            theRest = theRest[:trunc + len('-----END PGP SIGNATURE-----')]
            lastSignature = False
        return result

    def CreateListFromText (fromText) :
        """
            Creates a list of the GeekCredit class instances
            from the string source
        """
        myCredits = string.split(fromText, 'Geek Credit 0.3')
        toCreate = []
        for text in myCredits :
            if len(text) == 0 :
                continue
            text = 'Geek Credit 0.3' + text
            credit = GeekCredit(text)
            toCreate.append(credit)
        return toCreate
    CreateListFromText = staticmethod(CreateListFromText)

    def getText(self) :
        return self.text

class GeekCreditFile :
    """
        This instances of this class are the storage for Geek Credits
        This might be a database interface in the future, now this is
        just a file
    """
    def __init__ (self, filename) :
        self.filename = filename
        self.CreditsDict = {}
        self.wasChanged = False
        try :
            f = file(self.filename)
        except IOError, (errno, strerror) :
            if errno == 2 :
                print 'File ' + filename + ' does not exist, creating'
                fd = os.open(filename, os.O_CREAT)
                os.close(fd)
                try :
                    f = file(self.filename)
                except :
                    raise
            else :
                raise
        try :
            myCredits = f.read()
            f.close()
        except :
            raise
        if not len(myCredits) :
            return
        toAdd = GeekCredit.CreateListFromText(myCredits)
        if not len(toAdd) :
            # The file is empty yet. This is OK
            return
        for credit in toAdd :
            issuer = credit.getIssuer()
            if issuer in self.CreditsDict.keys() :
                self.CreditsDict[issuer].append(credit)
            else :
                self.CreditsDict[issuer] = [credit]
    def getItems (self) :
        return self.CreditsDict.items()
    def getKeys (self) :
        return self.CreditsDict.keys()
    def getDict (self) :
        return self.CreditsDict
    def removeCredits (self, myCredits) :
        for cur in myCredits :
            # If this credit is in our pocket
            if cur.getIssuer() in self.CreditsDict.keys() :
                self.wasChanged = True
                self.CreditsDict[cur.getIssuer()].remove(cur)

    def hasDuplicates (self, credit) :
        """
            Returns the number of duplicates
            for the GeekCredit parameter and
            file snapshot
        """
        myId = credit.getId()
        myComm   = credit.getCommunity()
        myIssuer = credit.getIssuer()
        gotDuplicates = 0
        for k in self.CreditsDict.values() :
            for j in k :
                if j.getId() == myId and j.getIssuer() == myIssuer :
                    gotDuplicates = gotDuplicates + 1
            # 2 duplicates is enough in any case
            if gotDuplicates == 2 :
                break
        return gotDuplicates

    def getDuplicateAttacker (self, credit, threshold = 2) :
        """
            Returns the attacker name
            The counterfeited credits: self, credit.
            The threshold is the number of means the
        """
        myId     = credit.getId()
        myComm   = credit.getCommunity()
        myIssuer = credit.getIssuer()
        gotDuplicates = 0
        for k in self.CreditsDict.values() :
            for j in k :
                if j.getId() == myId and j.getCommunity() == myComm and \
                   j.getIssuer() == myIssuer :
                    gotDuplicates = gotDuplicates + 1
                    if gotDuplicates == threshold :
                        # Get the first bogus transaction initiator
                        attacker = credit.findDifference(j)
                        return attacker
        return "Could not find attacker, try text editor"

    def generateUid (self) :
        while True :
            uid = str(random.randint(100000000, 900000000))
            found = False
            for k in self.CreditsDict.values() :
                for j in k :
                    if j.getId() == uid :
                        found = True
                        break;
                if found :
                    break
            if not found :
                break
        return uid

    def addCredits (self, myCredits) :
        for cur in myCredits :
            issuer = cur.getIssuer()
            self.wasChanged = True
            if issuer in self.CreditsDict.keys() :
                self.CreditsDict[issuer].append(cur)
            else :
                self.CreditsDict[issuer] = [cur]

    def savePocket (self) :
        """
            Saves current pocket data to file
        """
        if not self.wasChanged :
            return
        try :
            f = file(self.filename, "w+b")
            for issuer, myCredits in self.CreditsDict.items() :
                for credit in myCredits :
                    lines = string.split(credit.getText(), '\n')
                    for line in lines :
                        f.write(line + os.linesep)
            f.close()
        except IOError, (errno, strerror):
            print "I/O error(%s): %s" % (errno, strerror)
            print "Failed to save pocket, saving to terminal"
            for issuer, myCredits in self.CreditsDict.items() :
                for credit in myCredits :
                    print credit.getText()

class GeekCreditPocket :
    def __init__ (self, filename) :
        """
            Currently this contsructor reads pocket and history data to RAM on
            instantiation
        """
        self.passPhrase = ""
        self.trustedOnly = False
        try :
            # Reading config file
            config = ConfigParser.ConfigParser()
            config.readfp(open(filename))
            self.name   = config.get("General", "name")
            pocketname  = config.get("General", "pocket")
            historyname = config.get("General", "history")
            ratingsname = config.get("General", "ratings")
            communities = config.get("General", "communities")
            self.storePassPhrase = config.get("General", "store_passphrase")
            self.currentCommunity = int(config.get("General", "current_community"))
        except :
            raise Exception('Failed parsing config file')

        # Parsing optional parameters
        if config.has_option("General", "passphrase") :
            self.passPhrase = config.get("General", "passphrase")
        if config.has_option("Ratings", "trusted_only") :
            if config.get("Ratings", "trusted_only") == "yes" :
                self.TrustedOnly = True
        self.KeyServer = ""
        if config.has_option("General", "keyserver") :
            self.KeyServer = config.get("General", "keyserver")

        try :
            # Pocket file used to store Geek Credits
            self.pocketFile = GeekCreditFile(pocketname)
            # Check if this credit belongs to us
            for seller in self.pocketFile.getDict().values() :
                for credit in seller :
                    lastName = credit.getLastName()
                    if lastName == "" :
                        raise Exception('Bad credit in the pocket')
                    if lastName != self.name :
                        raise Exception('Bad credit in the pocket belongs to ' \
                                        + lastName)

            # File used to store redeemed and issued Geek Credits
            self.historyFile = GeekCreditFile(historyname)
        except IOError, (errno, strerror) :
            raise Exception('Unable to read pocket or history file')
        except Exception, list :
            raise Exception(str(list.args))

        self.badIssuers = []
        self.trustedIssuers = []
        try :
            f = file(ratingsname)
        except IOError, (errno, strerror) :
            if errno == 2 :
                print 'File ' + ratingsname + ' does not exist, creating'
                fd = os.open(ratingsname, os.O_CREAT)
                os.close(fd)
                try :
                    f = file(ratingsname)
                except Exception, list :
                    raise Exception(str(list.args))
            else :
                raise Exception(strerror)

        for line in file(ratingsname) :
            good = ""
            bad = ""
            if len(line) :
                tmp = re.search('(?<=bad:).+', line)
                if tmp :
                    tmp2 = re.search('\w+.+', tmp.group(0))
                    if tmp2 :
                        bad = tmp2.group(0)
                    else :
                        continue
                else :
                    tmp = re.search('(?<=good:).+', line)
                    if tmp :
                        tmp2 = re.search('\w+.+', tmp.group(0))
                        if tmp2 :
                            good = tmp2.group(0)
                        else :
                            continue
                    else :
                        continue
                if bad :
                    if bad == self.name :
                        raise Exception("Should accept own Geek Credits, policy #3")
                    self.badIssuers.append(bad)
                if good :
                    self.trustedIssuers.append(good)
        self.trustedIssuers.append(self.name)

        if len(communities) :
            try :
                self.communities = map(int, communities.split(","))
            except :
                raise Exception("failed to parse communities section")

    def savePocket (self) :
        """
            Saves [updated] Geek Credits to persistant pocket and history storage
        """
        self.pocketFile.savePocket()
        self.historyFile.savePocket()

    def getIssuers (self) :
        """
            Returns all issuers of the Geek Credits in the pocket + myself
        """
        issuers = []
        # All issuers in the pocket
        for issuer, myCredits in self.pocketFile.getItems() :
            if myCredits :
                issuers.append(issuer)
        # I can issue Geek Credits too
        issuers.append(self.name)
        return issuers

    def getCreditsStats (self) :
        """
            Returns all issuers of the Geek Credits in the pocket and number
            of Geek Credits
        """
        issuers = []
        # All issuers in the pocket
        for issuer, myCredits in self.pocketFile.getItems() :
            if myCredits :
                issuers.append([issuer, len(myCredits)])
        return issuers

    def getGoodIssuers(self, issuers) :
        """
            Input - list of the issuers payer wants us to accept
            Output - list of the issuers we do accept from this list
        """
        for cur in self.badIssuers :
            if cur in issuers :
                # According to the policy #7
                issuers.remove(cur)
        return issuers

    def signCredits(self, seller, issuers, creditsNeeded, passPhrase = "") :
        """
            Input: seller name, issuers accpeted, number of Geek Credits needed
            Output: list of signed Geek Credits ready to by given to seller,
                    errors string
            Side effect: this Geek Credits move from pocket to history
        """
        if seller == self.name :
            return [], 'Bad seller: myself'
        # First, pick the amount of Geek Credits we need
        creditsToGive = []
        creditsNow = 0
        j = 0
        if not seller in issuers :
            # According to the policy #3, seller MUST accept own Geek Credits
            issuers.append(seller)
        # According to the policy #4, we will give seller's own Geek Credits
        # first
        if seller in self.pocketFile.getDict() :
            if len(self.pocketFile.getDict()[seller]) :
                for credit in self.pocketFile.getDict()[seller] :
                    creditsToGive.append(credit)
                    creditsNow = creditsNow + 1
        while creditsNow < creditsNeeded :
            pickedAnything = 0
            for k in self.pocketFile.getKeys() :
                if k not in issuers or k == seller:
                    # seller won't accept this issuer or it is the seller
                    # (we already have picked all redeemable Geek Credits)
                    continue
                # According to the policy #4
                if len(self.pocketFile.getDict()[k]) > j :
                    cr = self.pocketFile.getDict()[k][j]
                    if int(cr.getCommunity()) != self.currentCommunity:
                        continue
                    creditsToGive.append(cr)
                    creditsNow = creditsNow + 1
                    pickedAnything = 1
            if pickedAnything == 0 :
                break
            j = j + 1
        # If we are out of Geek Credits, let's issue some new
        issued = []
        for k in range(creditsNeeded - creditsNow) :
            issued.append(GeekCredit("", \
                        self.historyFile.generateUid(), self.currentCommunity))
        creditsToGive = creditsToGive + issued

        if passPhrase == "" :
            passPhrase = self.passPhrase
        if self.storePassPhrase == 'no' :
            # No pass phrase option was set
            passPhrase = ""

        # Now sign the Geek Credits
        for cur in creditsToGive :
            try :
                cur.signTransaction(self, seller, passPhrase)
            except :
                return [], 'Failed to sign credit ' + cur.getId()

        # Remove the Geek Credits we are about to give from the pocket
        self.pocketFile.removeCredits(creditsToGive)

        # Move issued Geek Credits to the history file
        # according to the policy #5
        self.historyFile.addCredits(issued)
        return creditsToGive, ''

    def acceptCredits (self, stringCredits) :
        """
            Input: a list of Geek Credits to accept to the pocket
            Output: number of Geek Credits to accept, number accpeted, string errors
            Side effect: adds all valid Geek Credits to the pocket
        """
        try :
            myCredits = GeekCredit.CreateListFromText(stringCredits)
        except :
            return 0, 0, 'Malformed credit was detected.'
        if not len(myCredits) :
            return 0, 0, 'No Geek Credits'
        creditsAccepted = 0
        # For each credit
        result = ''
        for cur in myCredits :
            issuer = cur.getIssuer()
            if issuer in self.badIssuers :
                # We were given a bad Credit
                result = result + 'Bad issuer ' + issuer + '\n'
                continue
            if self.trustedOnly :
                if issuer not in self.trustedIssuers :
                    result = result + 'Issuer not in trusted list ' + \
                             issuer + '\n'
                    continue

            if int(cur.getCommunity()) not in self.communities :
                result = result + 'Bad community ' + cur.getCommunity() + '\n'
                continue

            # We cannot accept this credit, the name
            # does not match, go to next credit
            lastName = cur.getLastName()
            if lastName != self.name :
                result = result + 'Bad credit, it is for ' + lastName + \
                                  ' ,not ' + self.name + '\n'
                continue

            # Verify the signatures, according to policy #6
            res = cur.verifySignatures(self.KeyServer)
            if res != "" :
                result = result + res + '\n'
                continue

            # Check if there is a duplicate in the pocket,
            # go to next credit if found, according to policy #7
            if self.pocketFile.hasDuplicates(cur) :
                result = result + 'Received a duplicate !!!' + '\n'
                attacker = self.pocketFile.getDuplicateAttacker(cur, 1)
                result = result + "Investigation result: " + attacker
                continue

            # Redeem this credit if we are the issuer
            #    according to policy #3
            if issuer == self.name :
                # Check if there is a duplicate in the history
                if self.historyFile.hasDuplicates(cur) > 1 :
                    attacker = self.historyFile.getDuplicateAttacker(cur)
                    result = result + "Received a duplicate to redeem!!!\n"
                    result = result + "Investigation result: " + attacker
                    continue
                # According to the policy #5
                self.historyFile.addCredits([cur])
                creditsAccepted = creditsAccepted + 1
                continue
            else :
                # Check if there is a duplicate in the history
                if self.historyFile.hasDuplicates(cur) :
                    attacker = self.pocketFile.getDuplicateAttacker(cur, 1)
                    result = result + "Received a duplicate !!!" + '\n'
                    result = result + "Investigation result: " + attacker
                    continue

            # Append this credit to the pocket
            self.pocketFile.addCredits([cur])
            creditsAccepted = creditsAccepted + 1
        return creditsAccepted, len(myCredits), result

def gpgVerifySignature (text, signature, name) :
    """
        Input: text to verify, signature, the signer's ID
        Output: True if signature OK, False otherways
    """
    try :
        if os.name == "posix" :
            tmpname = tempfile.mktemp()
            tmpname2 = tempfile.mktemp()
            out = file(tmpname, "w+b")
            out2 = file(tmpname2, "w+b")
            myHash = sha.new(text).digest()
            out.write(myHash)
            out.flush()
            out2.write('-----BEGIN PGP SIGNATURE-----' + signature)
            out2.flush()
        else :
            try :
                os.remove("tmpfile3.txt")
            except :
                # We don't care
                pass
            try :
                os.remove("tmpfile4.txt")
            except :
                # We don't care
                pass
            tmpname = "tmpfile3.txt"
            tmpname2 = "tmpfile4.txt"
            out = file(tmpname, "w+b")
            out2 = file(tmpname2, "w+b")
            myHash = sha.new(text).digest()
            out.write(myHash)
            out.flush()
            out2.write('-----BEGIN PGP SIGNATURE-----' + signature)
            out2.flush()
            out.close()
            out2.close()
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't verify signature"
        return False
    # Run GPG to verify the signature
    result = os.system("gpg -q --verify " + tmpname2 + " " + tmpname + \
                        " > tempfile2.txt 2>&1") != 0
    try :
        out.close()
        out2.close()
        os.remove(tmpname)
        os.remove(tmpname2)
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't remove temp files, please remove manually"
    if result :
        try :
            os.remove("tempfile2.txt")
        except IOError, (errno, strerror):
            print "I/O error(%s): %s" % (errno, strerror)
            print "Can't remove temp files, please remove manually"
        return False
    # OK, the signature is good. Lets check
    # if it is a signature of the right person
    try :
        # This temporary file is a log that does not have any
        # useful information inside.
        logfile = file("tempfile2.txt", "rb")
        log = logfile.read()
        start = log.find("from \"")
        end = log.rfind("\"")
        if start == -1 or end == -1 or end == start + 5 :
            print 'Failed to verify credit'
            return False
        signername = log[start + 6:end]
        if name != signername :
            print 'Failed to verify credit'
            print 'signed by ' + signername + ' not ' + name
            return False
        logfile.close()
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't verify signature"
        return False
    # Clean up the log file
    try :
        os.remove("tempfile2.txt")
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't remove temp files, please remove manually"
    if os.name != "posix" :
        try :
            os.remove("tmpfile3.txt")
        except :
            pass
        try :
            os.remove("tmpfile4.txt")
        except :
            pass
    return True

def gpgHasKey (name, keyserver) :
    ending = ""
    if os.name == "posix" :
        ending = ">>gpg.log 2>&1"
    if os.system("gpg -q --list-keys \"" + name + "\"" + ending) == 0 :
        return True
    if keyserver != "" :
        if os.system("gpg -q --keyserver " + keyserver + \
                    " --recv-keys \"" + name + "\"" + ending) == 0 :
            return True

    return False

def gpgMakeSignature (text, signer, passPhrase) :
    """
        Input - text to sign (SHA1 hash of the Geek Credit), signer ID
        Output - signature made by gpg -a
        Returns "" if signing failed
    """
    try :
        if os.name == "posix" :
            tmpname = tempfile.mktemp()
            out = file(tmpname, "w+b")
            out.write(text)
            out.flush()
        else :
            try :
                os.remove("tmpfile2.txt")
            except :
                # We don't care
                pass
            tmpname = "tmpfile2.txt"
            out = file(tmpname, "w+b")
            out.write(text)
            out.close()
        try :
            os.remove("tmpfile.txt")
        except :
            # We don't care
            pass
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't sign"
        return ""
    # In linux, lets redirect gpg output to the log file
    if os.name == "posix" :
        ending = ">>gpg.log 2>&1"
    else :
        ending = ""
    if passPhrase == "" :
        gpgString = "gpg -q -o tmpfile.txt -u \"" + signer + \
                    "\" -a -b " + tmpname + " " + ending
    else :
        if os.name == "posix" :
            gpgString = "echo \"" + passPhrase + \
                        "\" | gpg -q --passphrase-fd=0 -o tmpfile.txt -u \"" + \
                        signer + "\" -a -b " + tmpname + " " + ending
        else :
            if os.system("echo " + passPhrase + ">tmppass.txt") != 0 :
                print "Failed running GPG"
                return ""
            gpgString = "type tmppass.txt | gpg -q --passphrase-fd=0 -o tmpfile.txt -u \"" + \
            signer + "\" -a -b " + tmpname + " " + ending

    # Run GPG to sign the Geek Credit
    if os.system(gpgString) != 0 :
        print "Failed running GPG"
        return ""
    try :
        inp = file("tmpfile.txt")
        signed = inp.read()
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't sign"
        return ""
    try :
        inp.close()
        out.close()
        os.remove(tmpname)
        os.remove("tmpfile.txt")
    except IOError, (errno, strerror):
        print "I/O error(%s): %s" % (errno, strerror)
        print "Can't remove temp files, please remove manually"
    if os.name != "posix" :
        try :
            os.remove("tmppass.txt")
        except :
            pass
    return signed
