#!/bin/bash
# This is the test for all Geek Credit pocket features
# It may also be used as a User Stories style requirements doc

echo Checking if we have 3 dummy users, if not, use
echo gpg --gen-keys
echo to generate 3 signature keys for dummy users
gpg --list-keys "alice <alice@wonderland.ru>" >/dev/null 2>&1 || { 
echo "no alice" 
exit
}
gpg --list-keys "bobby <bobby@wonderland.ru>" >/dev/null 2>&1 || {
echo "no bobby" 
exit
}
gpg --list-keys "catty <catty@wonderland.ru>" >/dev/null 2>&1 || {
echo "no catty"
exit
}
echo " Done."

# create empty pocket and history files
rm -f hist.txt pocket.txt
touch hist.txt pocket.txt

echo User alice wants to give 2 Geek Credits to user bobby
./GCPocket.py --GetCredits="2,bobby_<bobby@wonderland.ru>,alice_<alice@wonderland.ru>" -fgk_alice.cfg>to_bobby.txt || {
echo "failed to generate credits"
exit
}
echo " Done."

# Back up the history file of alice
mv hist.txt hist_alice.txt
touch hist.txt

echo Geek Credit pocket of alice and catty should not accept Geek Credits designated to bobby
./GCPocket.py -fgk_alice.cfg --AcceptCredits=to_bobby.txt>/dev/null
wc -l pocket.txt | grep 0 > tst.log|| {
echo 'Accept wrong'
exit
}
./GCPocket.py -fgk_catty.cfg --AcceptCredits=to_bobby.txt>/dev/null
wc -l pocket.txt | grep 0 >>tst.log|| {
echo 'Accept wrong'
exit
}
echo " Done."

echo Checking if bobby would accept this credits
./GCPocket.py -fgk_bobby.cfg --AcceptCredits=to_bobby.txt | grep "Accepted 2 from 2">>tst.log || {
echo 'Failed to accept'
exit
}
echo " Done."

echo User bobby wants to give 3 Geek Credits to user catty
./GCPocket.py --GetCredits="3,catty_<catty@wonderland.ru>,bobby_<bobby@wonderland.ru>,alice_<alice@wonderland.ru>" -fgk_bobby.cfg>to_catty.txt || {
echo "failed to generate credits"
exit
}
echo " Done."

# Back up the history file of alice
mv hist.txt hist_bobby.txt
touch hist.txt

echo Checking if catty would accept this credits
./GCPocket.py -fgk_catty.cfg --AcceptCredits=to_catty.txt | grep "Accepted 3 from 3">/dev/null || {
echo 'Failed to accept'
exit
}
echo " Done."

echo Check if catty has right number of issuers
./GCPocket.py -fgk_catty.cfg --GetIssuers | wc -l |grep 3 >/dev/null|| {
echo 'Wrong number of issuers'
exit
}
echo " Done."

echo User catty wants to give 4 Geek Credits to user alice
./GCPocket.py --GetCredits="4,alice_<alice@wonderland.ru>,catty_<catty@wonderland.ru>,bobby_<bobby@wonderland.ru>,alice_<alice@wonderland.ru>" -fgk_catty.cfg>to_alice.txt || {
echo "failed to generate credits"
exit
}
echo " Done."

# Back up catty files
mv hist.txt hist_catty.txt
# Restore alice files
mv hist_alice.txt hist.txt

echo Giving 4 Credits to alice
./GCPocket.py -fgk_alice.cfg --AcceptCredits=to_alice.txt>/dev/null
cat pocket.txt |grep "Geek Credit"|wc -l | grep 2 >/dev/null|| {
echo "Something went wrong, whole test failed"
exit
}
echo " Done."
echo The test had finished successfully

